/*
Copyright  2005 The Regents of the University of California.
    All Rights Reserved.

    Permission to use, copy, modify, and distribute this software and its
    documentation for educational, research and non-profit purposes,
    without fee, and without a written agreement is hereby granted,
    provided that the above copyright notice, this paragraph and the
    following three paragraphs appear in all copies.

    Permission to incorporate this software into commercial products may
    be obtained by contacting the University of California. For
    information about obtaining such a license contact:

    Chrisanna Waldrop
    Copyright Officer
    805-893-7773
    waldrop@research.ucsb.edu

    This software program and documentation are copyrighted by The Regents
    of the University of California. The software program and
    documentation are supplied "as is", without any accompanying services
    from The Regents. The Regents does not warrant that the operation of
    the program will be uninterrupted or error-free. The end-user
    understands that the program was developed for research purposes and
    is advised not to rely exclusively on the program for any reason.

    IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
    FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
    INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
    ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
    ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF
    CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
    A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
    BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE
    MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

#ifndef MD_IGMP_H
#define MD_IGMP_H

#include <Ping.h>

#define IGMP_MEMBERSHIP_QUERY           0x11    /* membership query         */
#define IGMP_V1_MEMBERSHIP_REPORT       0x12    /* Ver  1 membership report */
#define IGMP_V2_MEMBERSHIP_REPORT       0x16    /* Ver  2 membership report */
#define IGMP_V3_MEMBERSHIP_REPORT       0x22    /* Ver  3 membership report */
#define IGMP_V2_LEAVE_GROUP             0x17    /* Leave-group message      */
#define IGMP_DVMRP                      0x13    /* DVMRP routing message    */
#define IGMP_PIM                        0x14    /* PIM routing message for PIM v1*/

enum IGMPV3_RecordType
{
	IGMPV3_MODE_IS_INCLUDE = 1,
    IGMPV3_MODE_IS_EXCLUDE = 2,
	IGMPV3_CHANGE_TO_INCLUDE_MODE = 3,
	IGMPV3_CHANGE_TO_EXCLUDE_MODE = 4,
	IGMPV3_ALLOW_NEW_SOURCES = 5,
	IGMPV3_BLOCK_OLD_SOURCES = 6
};

class MD_Igmp
{
public:
    struct IGMPHeader
    {
        unsigned char   igmp_type;
        unsigned char   igmp_code;
        unsigned short  igmp_cksum;
        struct in_addr  igmp_group;
    };

	struct IGMPV3MembershipReport
	{
		unsigned char   igmp_type;
		unsigned char   igmp_reserved1;
		unsigned short  igmp_cksum;
		unsigned short  igmp_reserved2;
		unsigned short  igmp_grouprecord_count;
	};

	struct IGMPV3GroupRecord
	{
		unsigned char   igmp_record_type;
		unsigned char   igmp_auxdata_len;
		unsigned short  igmp_source_count;
		struct in_addr  igmp_group;
	};

    enum QueryType
    {
        None,
        GeneralQuery,
        GroupQuery,
    };
    
    MD_Igmp( struct sockaddr_in& group );

    ~MD_Igmp() {}

    void ReceiveQuery( int timeoutMs, bool waitForMultipleQuery = true );

    bool IsResponseOfType( QueryType type );

	void SendMembershipReport2( sockaddr_in& group );

	void SendMembershipReport3( sockaddr_in&      group, 
		                        sockaddr_in&      source,
								IGMPV3_RecordType type );

	void SendLeaveGroup( sockaddr_in& group );

	void Receive( int timeoutMs, short port, bool waitForMultipleResponse = true );

	void JoinGroup( std::string& group );

    void LeaveGroup( std::string& group );

private:
    MD_Socket            m_socket;
 
    struct sockaddr_in   m_group;

    QueryType            m_response;
};

static unsigned __stdcall ReceiveQueryThreadFunc( void*  pThis )
{
	MD_Igmp* pIgmp = (MD_Igmp*)pThis;

	pIgmp->ReceiveQuery( 10000 );

	return 0;
}

#endif