/*
Copyright  2005 The Regents of the University of California.
    All Rights Reserved.

    Permission to use, copy, modify, and distribute this software and its
    documentation for educational, research and non-profit purposes,
    without fee, and without a written agreement is hereby granted,
    provided that the above copyright notice, this paragraph and the
    following three paragraphs appear in all copies.

    Permission to incorporate this software into commercial products may
    be obtained by contacting the University of California. For
    information about obtaining such a license contact:

    Chrisanna Waldrop
    Copyright Officer
    805-893-7773
    waldrop@research.ucsb.edu

    This software program and documentation are copyrighted by The Regents
    of the University of California. The software program and
    documentation are supplied "as is", without any accompanying services
    from The Regents. The Regents does not warrant that the operation of
    the program will be uninterrupted or error-free. The end-user
    understands that the program was developed for research purposes and
    is advised not to rely exclusively on the program for any reason.

    IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
    FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
    INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
    ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
    ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF
    CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
    A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
    BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE
    MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

#pragma warning (disable: 4786)

#include <Join.h>

#include <ws2tcpip.h>

/*
#define IP_ADD_SOURCE_MEMBERSHIP        15
#define IP_DROP_SOURCE_MEMBERSHIP       16

 
struct ip_mreq_source 
{
	struct in_addr           imr_multiaddr;
	struct in_addr			 imr_interface;
	struct in_addr           imr_sourceaddr;
};
*/

MD_Join::MD_Join( short portNumber )
 : m_socket(  SOCK_DGRAM, IPPROTO_UDP, INADDR_ANY, portNumber )
{
}

void MD_Join::JoinGroup( std::string& group )
{
    ip_mreq  mreq;
    mreq.imr_multiaddr.s_addr = inet_addr( group.c_str() );
    mreq.imr_interface.s_addr = htonl(INADDR_ANY);

    m_socket.SetOption( IPPROTO_IP, IP_ADD_MEMBERSHIP, (char*)&mreq, sizeof(ip_mreq) );
}

void MD_Join::LeaveGroup( std::string& group )
{
    ip_mreq  mreq;
    mreq.imr_multiaddr.s_addr = inet_addr( group.c_str() );
    mreq.imr_interface.s_addr = htonl(INADDR_ANY);

    m_socket.SetOption( IPPROTO_IP, IP_DROP_MEMBERSHIP, (char*)&mreq, sizeof(ip_mreq) );
}

void MD_Join::SubsribeChannel( std::string& group, std::string& source )
{
    ip_mreq_source  mreq;
    mreq.imr_multiaddr.s_addr = inet_addr( group.c_str() );
    mreq.imr_sourceaddr.s_addr = inet_addr( source.c_str() );
    mreq.imr_interface.s_addr = htonl(INADDR_ANY);

    m_socket.SetOption( IPPROTO_IP, IP_ADD_SOURCE_MEMBERSHIP, (char*)&mreq, sizeof(ip_mreq_source) );
}    


void MD_Join::UnsubscribeChannel( std::string& group, std::string& source )
{
    ip_mreq_source  mreq;
    mreq.imr_multiaddr.s_addr = inet_addr( group.c_str() );
    mreq.imr_sourceaddr.s_addr = inet_addr( source.c_str() );
    mreq.imr_interface.s_addr = INADDR_ANY;

    m_socket.SetOption( IPPROTO_IP, IP_DROP_SOURCE_MEMBERSHIP, (char*)&mreq, sizeof(ip_mreq_source) );
}

void MD_Join::Send( char* buffer, int size, sockaddr_in& address )
{
	m_socket.SendTo( buffer, size, &address );
}
   
void MD_Join::Receive( int timeoutMs, short port, bool waitForMultipleResponse )
{
    sockaddr_in source;

    char  buffer[4096];

    memset( &source, 0, sizeof( sockaddr_in ) );

    int waitInterval = timeoutMs;

    int timestamp = GetTickCount();

    while( waitInterval > 0 )
    {
        if( m_socket.IsDataAvailableForRead( waitInterval ) )
        {
            int bytes = m_socket.ReceiveFrom( buffer, 4096, &source );

            std::string  address( inet_ntoa( source.sin_addr ) );

            AddressMap::iterator iter = m_responseMap.find( address );

            if( iter != m_responseMap.end() )
            {
                (iter->second)++;
            }
            else
            {
                m_responseMap[address] = 1;
            }

            if( !waitForMultipleResponse )
            {
                break;
            }
        }

        waitInterval = timeoutMs - ( GetTickCount() - timestamp );
    }
}


int  MD_Join::GetResponseFromMemberCount()
{
    return m_responseMap.size();
}

void MD_Join::DisableLoopback()
{
	char flag = (char)0;

	m_socket.SetOption( IPPROTO_IP, IP_MULTICAST_LOOP, &flag, sizeof(char) );
}

void MD_Join::SetMulticastTtl( DWORD ttl )
{
	m_socket.SetOption( IPPROTO_IP, IP_MULTICAST_TTL, (char*)&ttl, sizeof(DWORD) );
}