/*
Copyright  2005 The Regents of the University of California.
    All Rights Reserved.

    Permission to use, copy, modify, and distribute this software and its
    documentation for educational, research and non-profit purposes,
    without fee, and without a written agreement is hereby granted,
    provided that the above copyright notice, this paragraph and the
    following three paragraphs appear in all copies.

    Permission to incorporate this software into commercial products may
    be obtained by contacting the University of California. For
    information about obtaining such a license contact:

    Chrisanna Waldrop
    Copyright Officer
    805-893-7773
    waldrop@research.ucsb.edu

    This software program and documentation are copyrighted by The Regents
    of the University of California. The software program and
    documentation are supplied "as is", without any accompanying services
    from The Regents. The Regents does not warrant that the operation of
    the program will be uninterrupted or error-free. The end-user
    understands that the program was developed for research purposes and
    is advised not to rely exclusively on the program for any reason.

    IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
    FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
    INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
    ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
    ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF
    CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
    A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
    BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE
    MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

#pragma warning (disable: 4786)

#include <PIMSM.h>

MD_Pimsm::MD_Pimsm()
: m_socket(  IPPROTO_IP )
{
	m_socket.Bind( IPPROTO_IP );
}

void MD_Pimsm::SendHello()
{
    sockaddr_in  dest;

    char buffer[4096];
    
    memset( buffer, 0, 4096 );    
    memset( &dest, 0, sizeof( sockaddr_in ) );

    int addr = inet_addr( ALL_PIM_ROUTERS_GROUP );

    if( addr != INADDR_NONE )
    {
        dest.sin_addr.s_addr = addr;
        dest.sin_family = AF_INET;
    
        PIMHeader* pimHeader = (PIMHeader*)buffer;
        
        pimHeader->pim_type = PIM_HELLO;
        pimHeader->pim_version = 2;
        
        HelloOption* holdTime = (HelloOption*)( buffer + sizeof(PIMHeader) );
        
        holdTime->hello_optionType = PIM_HOLDTIME_OPTION_TYPE;
        holdTime->hello_optionLength = PIM_HOLDTIME_OPTION_LEN;
        holdTime->hello_optionValue = 30; // 120 sec = 2min
        
        int bufferSize = sizeof(PIMHeader) + sizeof(HelloOption);
 
        // Calculate a checksum on the result
        pimHeader->pim_checksum = m_socket.CalculateCheckSum((short*)buffer, bufferSize);

        m_socket.SendTo( buffer, bufferSize, &dest );
    }
}


void MD_Pimsm::ReceiveRPSet( int timeoutMs )
{
    m_socket.SetIoctlForAllIP();

	sockaddr_in source;

    char  buffer[4096];
    
    memset( buffer, 0, 4096 );
    memset( &source, 0, sizeof( sockaddr_in ) );

    int waitInterval = timeoutMs;

    int timestamp = GetTickCount();

    while( waitInterval > 0 )
    {
        if( m_socket.IsDataAvailableForRead( waitInterval ) )
        {
            int bytes = m_socket.ReceiveFrom( buffer, 4096, &source );

            IPHeader* ipHeader = (IPHeader*)buffer;

            // Skip ahead to the PIM header within the IP packet
            unsigned short len = ipHeader->ip_hl * 4;
            PIMHeader* pimhdr = (PIMHeader*)(buffer + len);

            // TBD: DO size and checksum verification.
            if(    pimhdr->pim_type == PIM_CANDIDATE_RP_ADV
                && pimhdr->pim_version == 2  ) 
            {
                std::string address  = inet_ntoa( source.sin_addr );

                if( m_rpAdvtSet.find( address ) == m_rpAdvtSet.end() )
                {
                    m_rpAdvtSet.insert( address );
                }

            }
			else if(    pimhdr->pim_type == PIM_HELLO
                     && pimhdr->pim_version == 2 )
			{
				std::string address = inet_ntoa( source.sin_addr );
                
                if( m_helloResponseSet.find( address ) == m_helloResponseSet.end() )
                {
                    m_helloResponseSet.insert( address );
                }
			}
        }

        waitInterval = timeoutMs - ( GetTickCount() - timestamp );
	}
}

std::string MD_Pimsm::GetHelloResponderAddresses()
{
    std::string result;

    for( std::set<std::string>::iterator iter = m_helloResponseSet.begin();
         iter != m_helloResponseSet.end();
         iter++ )
    {
        result += (*iter);
        result += ", ";
    }

    return result;
}


std::string MD_Pimsm::GetRpAdvtAddresses()
{
    std::string result;

    for( std::set<std::string>::iterator iter = m_rpAdvtSet.begin();
         iter != m_rpAdvtSet.end();
         iter++ )
    {
        result += (*iter);
        result += ", ";
    }

    return result;
}