/*
Copyright  2005 The Regents of the University of California.
    All Rights Reserved.

    Permission to use, copy, modify, and distribute this software and its
    documentation for educational, research and non-profit purposes,
    without fee, and without a written agreement is hereby granted,
    provided that the above copyright notice, this paragraph and the
    following three paragraphs appear in all copies.

    Permission to incorporate this software into commercial products may
    be obtained by contacting the University of California. For
    information about obtaining such a license contact:

    Chrisanna Waldrop
    Copyright Officer
    805-893-7773
    waldrop@research.ucsb.edu

    This software program and documentation are copyrighted by The Regents
    of the University of California. The software program and
    documentation are supplied "as is", without any accompanying services
    from The Regents. The Regents does not warrant that the operation of
    the program will be uninterrupted or error-free. The end-user
    understands that the program was developed for research purposes and
    is advised not to rely exclusively on the program for any reason.

    IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
    FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
    INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
    ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
    ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF
    CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
    A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
    BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE
    MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

#ifndef MD_PIMSM_H
#define MD_PIMSM_H

#include <Ping.h>
#include <set>

#define PIM_HELLO                       0
#define PIM_REGISTER                    1
#define PIM_REGISTER_STOP               2
#define PIM_JOIN_PRUNE                  3
#define PIM_BOOTSTRAP                   4
#define PIM_ASSERT                      5
#define PIM_CANDIDATE_RP_ADV            8

#define IPPROTO_PIM                     103

#define ALL_PIM_ROUTERS_GROUP           "224.0.0.13"

#define PIM_HOLDTIME_OPTION_TYPE        1
#define PIM_HOLDTIME_OPTION_LEN         2

class MD_Pimsm // PIM Version 2
{
public:
    struct PIMHeader
    {
        unsigned char          pim_type:4;
        unsigned char          pim_version:4;
        unsigned char          pim_reserved;
        unsigned short         pim_checksum;
    };

    struct HelloOption
    {
        unsigned short         hello_optionType;
        unsigned short         hello_optionLength;
        unsigned short         hello_optionValue;
    };       
    
    MD_Pimsm();

    ~MD_Pimsm() {}

    void SendHello();

    void ReceiveRPSet( int timeoutMs );

    std::string GetHelloResponderAddresses();

    std::string GetRpAdvtAddresses();

private:
    MD_Socket     m_socket;

    std::set<std::string>   m_helloResponseSet;
    std::set<std::string>   m_rpAdvtSet;
};

#endif