/*
Copyright  2005 The Regents of the University of California.
    All Rights Reserved.

    Permission to use, copy, modify, and distribute this software and its
    documentation for educational, research and non-profit purposes,
    without fee, and without a written agreement is hereby granted,
    provided that the above copyright notice, this paragraph and the
    following three paragraphs appear in all copies.

    Permission to incorporate this software into commercial products may
    be obtained by contacting the University of California. For
    information about obtaining such a license contact:

    Chrisanna Waldrop
    Copyright Officer
    805-893-7773
    waldrop@research.ucsb.edu

    This software program and documentation are copyrighted by The Regents
    of the University of California. The software program and
    documentation are supplied "as is", without any accompanying services
    from The Regents. The Regents does not warrant that the operation of
    the program will be uninterrupted or error-free. The end-user
    understands that the program was developed for research purposes and
    is advised not to rely exclusively on the program for any reason.

    IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
    FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
    INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
    ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
    ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF
    CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
    A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
    BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE
    MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

#ifndef MD_PING_H
#define MD_PING_H

#include <Socket.h>

#include <map>


#define ICMP_ECHO_REPLY    0
#define ICMP_DEST_UNREACH  3
#define ICMP_TTL_EXPIRE    11
#define ICMP_ECHO_REQUEST  8

struct IPHeader 
{
    unsigned char          ip_hl:4;
    unsigned char          ip_v:4;
    unsigned char          ip_tos;
    unsigned short         ip_len;
    unsigned short         ip_id;
    unsigned short         ip_off;
    unsigned char          ip_ttl;
    unsigned char          ip_p;
    unsigned short         ip_sum;
    unsigned int           ip_src;
    unsigned int           ip_dst;
};

class MD_Ping
{
public:
    struct ICMPHeader
    {
        unsigned char      icmp_type;
        unsigned char      icmp_code;
        unsigned short     icmp_cksum;
        unsigned short     icmp_id;
        unsigned short     icmp_seq;
    };
    
    MD_Ping();

    ~MD_Ping() {}

    void Send( std::string address );

    void Receive( int timeoutMs, bool waitForMultipleResponse = true );

    bool DidReceiveResponse()
    {
        return ( m_responseMap.size() == 0 ? false : true );
    }

private:
    MD_Socket   m_socket;

    int         m_seqNo;

    typedef std::map<std::string,int>  AddressMap;

    AddressMap  m_responseMap;
};

#endif