/*
Copyright  2005 The Regents of the University of California.
    All Rights Reserved.

    Permission to use, copy, modify, and distribute this software and its
    documentation for educational, research and non-profit purposes,
    without fee, and without a written agreement is hereby granted,
    provided that the above copyright notice, this paragraph and the
    following three paragraphs appear in all copies.

    Permission to incorporate this software into commercial products may
    be obtained by contacting the University of California. For
    information about obtaining such a license contact:

    Chrisanna Waldrop
    Copyright Officer
    805-893-7773
    waldrop@research.ucsb.edu

    This software program and documentation are copyrighted by The Regents
    of the University of California. The software program and
    documentation are supplied "as is", without any accompanying services
    from The Regents. The Regents does not warrant that the operation of
    the program will be uninterrupted or error-free. The end-user
    understands that the program was developed for research purposes and
    is advised not to rely exclusively on the program for any reason.

    IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
    FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
    INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
    ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
    ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF
    CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
    A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
    BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE
    MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

#ifndef MD_SOCK_H
#define MD_SOCK_H

#include <winsock2.h>
#include <string>
#include <sstream>

class MD_Socket
{
public:    
    MD_Socket( int type, int protocol, int localAddress = INADDR_ANY, short localport = 10000 );

    MD_Socket( int protocol );

    ~MD_Socket();

    int SendTo( char* buffer, int size, sockaddr_in* toAddr );

    int ReceiveFrom( char* buffer, int len, sockaddr_in* fromAddr );

    void Bind( sockaddr_in* name );

	void Bind( int proticol );

    bool IsDataAvailableForRead( int timeoutMs = 0 );

    void SetOption( int level, int optname, char* optionValue, int optionSize );

    static short CalculateCheckSum( short* buffer, int size);

	void SetIoctlForIGMP();

	void SetIoctlForAllIP();

private:
    SOCKET  m_socket;
};

class MD_SocketException
{
public:
    MD_SocketException( int errorCode, std::string methodName )
    : m_errorCode( errorCode ),
      m_methodName( methodName )
    {
    }

    std::string Display()
    {
        std::stringstream error;
        
        error << "Exception while running test : " << m_methodName << "failed with error code : " << m_errorCode << std::endl;

        return error.str();
    }


private:
    int                   m_errorCode;
    std::string           m_methodName;
};

class MD_RawSocketException
{
public:
    MD_RawSocketException( int errorCode )
    : m_errorCode( errorCode )
    {
    }

    std::string Display()
    {
        std::stringstream error;
        
        error << "Raw socket usage failed with error : " << m_errorCode 
              << ". You might not have administrative privilege. "
              << std::endl;

        return error.str();
    }


private:
    int                   m_errorCode;
};

class MD_WinsockInit
{
public:	
	MD_WinsockInit();

	~MD_WinsockInit();
};
    
#endif