/*
Copyright  2005 The Regents of the University of California.
    All Rights Reserved.

    Permission to use, copy, modify, and distribute this software and its
    documentation for educational, research and non-profit purposes,
    without fee, and without a written agreement is hereby granted,
    provided that the above copyright notice, this paragraph and the
    following three paragraphs appear in all copies.

    Permission to incorporate this software into commercial products may
    be obtained by contacting the University of California. For
    information about obtaining such a license contact:

    Chrisanna Waldrop
    Copyright Officer
    805-893-7773
    waldrop@research.ucsb.edu

    This software program and documentation are copyrighted by The Regents
    of the University of California. The software program and
    documentation are supplied "as is", without any accompanying services
    from The Regents. The Regents does not warrant that the operation of
    the program will be uninterrupted or error-free. The end-user
    understands that the program was developed for research purposes and
    is advised not to rely exclusively on the program for any reason.

    IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
    FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
    INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
    ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
    ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF
    CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
    A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
    BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE
    MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

#pragma warning (disable: 4786)

#include <process.h>

#include <Ping.h>
#include <Join.h>
#include <PIMSM.h>
#include <IGMP.h>
#include <iostream>


#define MULTICAST_BEACON_GROUP       "233.4.200.19"
#define MULTICAST_BEACON_GROUP_PORT  10002

#define MULTICAST_ALL_HOST_GROUP     "224.0.0.1"

#define ALL_MULTICAST_ROUTER_GROUP   "224.0.0.2"
#define ALL_SSM_ROUTER_GROUP         "224.0.0.22"

#define SSM_VIDEO_GROUP              "232.64.133.120"
#define SSM_VIDEO_SOURCE             "63.105.122.29"
#define SSM_VIDEO_GROUP_PORT         8020

#include <ws2tcpip.h>

int main( int argc, char* argv[] )
{
    try
    {
		MD_WinsockInit  winsock;
		std::cout << "*****************************************" << std::endl;
        
        std::cout << "*     Multicast Detective Beta          *" << std::endl;
		std::cout << "*****************************************" << std::endl;

		std::cout << std::endl << std::endl;

		for( ;; )
		{
			bool hostSupportsV2 = false;
			bool hostSupportsV3 = false;
			bool routerSupportsV2 = false;
			bool routerSupportsV3 = false;
			bool routerSupportsPIM = false;
			bool routerPIMActive = false;
			bool asmRoutingSuccess = false;
			bool ssmRoutingSuccess = false;
			bool firstHopV2 = false;
			bool firstHopV3 = false;
			std::string routerAddress;

			std::cout << "Running 8 Tests : \n";

			// Test 1 : Join Beacon Group
			try
			{
				std::cout << "	1.  Joining NLAR Multicast Beacon Group 232.64.133.120, port 10002 \n";


				MD_Join  join(MULTICAST_BEACON_GROUP_PORT);
        
				join.JoinGroup( std::string( MULTICAST_BEACON_GROUP ) );

				join.Receive( 2000, MULTICAST_BEACON_GROUP_PORT );

				if( join.GetResponseFromMemberCount() > 0 )
				{
					std::cout << "Test succeeded:   received response from " << join.GetResponseFromMemberCount()
							  << " members \n";
					asmRoutingSuccess = true;
				}
				else
				{
					std::cout << "Test failed:  did not get reponse from any member \n";
				}

				join.LeaveGroup( std::string( MULTICAST_BEACON_GROUP ) );

				hostSupportsV2 = true;
			}
			catch( MD_SocketException& exp )
			{
				std::cout << "Exception while joining Beacon Group:" << exp.Display() << std::endl;
			}
        

			//Test 2 : Host support for IGMP V3
			
			try
			{
				std::cout << "	2.  Joining SSM Video Group 232.4.200.19, 63.105.122.29 port 8020 \n";

				Sleep( 2000 );

				MD_Join   join( SSM_VIDEO_GROUP_PORT );

				try
				{
					// Using an arbitrary group in SSM range
					join.SubsribeChannel( std::string( SSM_VIDEO_GROUP ), std::string( SSM_VIDEO_SOURCE ) );

					join.Receive( 2000, SSM_VIDEO_GROUP_PORT );

					if( join.GetResponseFromMemberCount() > 0 )
					{
						std::cout << "Test succeeded:   received response from " << join.GetResponseFromMemberCount()
								  << " members \n";

						ssmRoutingSuccess = true;
					}
					else
					{
						std::cout << "Test failed:  did not get reponse from any member \n";
					}

					join.UnsubscribeChannel( std::string( SSM_VIDEO_GROUP ), std::string( SSM_VIDEO_SOURCE ) );
					
					hostSupportsV3 = true;
				}
				catch( MD_SocketException& exp )
				{
					std::cout << "Exception while joining SSM Group:" << exp.Display();
                    std::cout << "The host does not support SSM <IGMPv3> \n";
				}
			}
			catch( MD_SocketException& exp )
			{
				std::cout << "Exception while joining SSM Group:" << exp.Display() << std::endl;
			}
			
        
			//Test 3: First hop Router capability
			try
			{
				std::cout << "	3.  Checking presence of multicast capable router in the neighborhood \n";

				MD_Ping  ping;

				ping.Send( std::string(ALL_MULTICAST_ROUTER_GROUP) );

				ping.Receive( 2000, false );

				if( ping.DidReceiveResponse() )
				{
					routerSupportsV2 = true;

                    //std::cout << "Test succeeded:  first-hop router responded to 'all multicast router group' ping \n";
				}
                else
                {
                    //std::cout << "Test failed:  no router responded to ping \n";
                } 
			}
			catch( MD_RawSocketException& exp )
			{
				std::cout << exp.Display();
			}
            catch( MD_SocketException& exp )
			{
				std::cout << exp.Display();
			}

        
			try
			{            
				std::cout << "	4.  Checking presence of SSM capable router in the neighborhood \n";

				MD_Ping  ping;

				ping.Send( std::string(ALL_SSM_ROUTER_GROUP) );

				ping.Receive( 2000, false );

				if( ping.DidReceiveResponse() )
				{
					routerSupportsV3 = true;
                    //std::cout << "Test succeeded:  first-hop router responded to 'all ssm router group' ping \n";
				}
                else
                {
                    //std::cout << "Test failed:  no router responded to ping \n";
                }
			}
			catch( MD_RawSocketException& exp )
			{
				std::cout << exp.Display();
			}



			// Test 4: Detecting first hop IGMP combined unit
			try
			{
				std::cout << "	5.  Detecting if there is IGMPv2 capability to first-hop router \n";
				
				sockaddr_in  group;
				memset( &group, 0, sizeof(sockaddr_in) );

				group.sin_addr.s_addr = inet_addr( MULTICAST_BEACON_GROUP );
				group.sin_family = AF_INET;

		
				MD_Igmp  igmp( group );

				igmp.SendMembershipReport2( group );

				Sleep( 2000 );

				igmp.SendLeaveGroup( group );

				igmp.ReceiveQuery( 1000 );

				if( igmp.IsResponseOfType( MD_Igmp::GeneralQuery ) ||
					igmp.IsResponseOfType( MD_Igmp::GroupQuery ) )
				{
					firstHopV2 = true;

                    std::cout << "Test succeeded:  received IGMPv2 response from the first-hop router \n";
				}
                else
                {
                    std::cout << "Test failed:  no IGMPv2 response received from the router \n";
                }
            
			}
			catch( MD_RawSocketException& exp )
			{
				std::cout << exp.Display();
			}
            catch( MD_SocketException& exp )
			{
				std::cout << exp.Display();
			}

			// Test 5: Detecting first hop IGMP combined unit
			try
			{
				std::cout << "	6.  Detecting if there is IGMPv3 capability to first-hop router \n";
				
				Sleep( 5000 );
				
				sockaddr_in  group;
				memset( &group, 0, sizeof(sockaddr_in) );

				group.sin_addr.s_addr = inet_addr( SSM_VIDEO_GROUP );
				group.sin_family = AF_INET;

				sockaddr_in  source;
				memset( &source, 0, sizeof(sockaddr_in) );

				source.sin_addr.s_addr = inet_addr( SSM_VIDEO_SOURCE );
				source.sin_family = AF_INET;
				
				MD_Igmp  igmp( group );

				igmp.SendMembershipReport3( group, source, IGMPV3_ALLOW_NEW_SOURCES );
				
				Sleep( 2000 );

				igmp.SendMembershipReport3( group, source, IGMPV3_BLOCK_OLD_SOURCES );

				igmp.ReceiveQuery( 1000 );

				if( igmp.IsResponseOfType( MD_Igmp::GeneralQuery ) ||
					igmp.IsResponseOfType( MD_Igmp::GroupQuery ) )
				{
					firstHopV3 = true;

                    std::cout << "Test succeeded:  received IGMPv3 response from the first-hop router \n";
				}
                else
                {
                    std::cout << "Test failed:  no IGMPv3 response received from the router \n";
                }
			}
			catch( MD_RawSocketException& exp )
			{
				std::cout << exp.Display();
			}
            catch( MD_SocketException& exp )
			{
				std::cout << exp.Display();
			}

			//Test 5: Detecting PIM-SM capability
			try
			{            
				std::cout << "	7.  Detecting presence of PIM-SM capable router \n";
				
				MD_Ping  ping;

				ping.Send( std::string(ALL_PIM_ROUTERS_GROUP) );

				ping.Receive( 2000, false );

				if( ping.DidReceiveResponse() )
				{
					routerSupportsPIM = true;
                    //std::cout << "Test succeeded:  first-hop router responded to 'all pim routers group' ping \n";
				}
                else
                {
                    //std::cout << "Test failed:  no router responded to PIM ping \n";
                }
			}
			catch( MD_RawSocketException& exp )
			{
				std::cout << exp.Display();
			}
            catch( MD_SocketException& exp )
			{
				std::cout << exp.Display();
			}


			try
			{
				std::cout << "	8.  Detecting if first-hop router knows about an RP \n";	
				
				MD_Pimsm   pimsm;

				pimsm.SendHello();

				pimsm.ReceiveRPSet( 5000 );

				if( pimsm.GetHelloResponderAddresses().size() != 0 )
				{
					routerPIMActive = true;
					routerAddress = pimsm.GetHelloResponderAddresses();
                    std::cout << "Test succeeded:  received responses to Hello Message \n";
				}
                else if( pimsm.GetRpAdvtAddresses().size() != 0 )
                {
                    routerPIMActive = true;
					routerAddress = pimsm.GetHelloResponderAddresses();
                    std::cout << "Test succeeded:  first-hop router responded with RP \n";
                }
                else
                {
                    std::cout << "Test failed:  no router responded to PIM-SM Hello \n";
                }
			}
			catch( MD_RawSocketException& exp )
			{
				std::cout << exp.Display();
			}
            catch( MD_SocketException& exp )
			{
				std::cout << exp.Display();
			}
            
			std::cout << std::endl << std::endl;

			std::cout << "*****************************************" << std::endl;
			std::cout << "*     Multicast Detective Report        *" << std::endl;
			std::cout << "*****************************************" << std::endl;

			if( hostSupportsV2 )
			{
				std::cout << "=> Host Operating System is capable of Any Source Multicast (IGMP v2) \n";
			}

			if( hostSupportsV3 )
			{
				std::cout << "=> Host Operating System is capable of Source Specific Multicast (IGMP v3) \n";
			}

			if( routerSupportsV2 )
			{
				std::cout << "=> First hop router supports Any Source Multicast (IGMP v2). "
					         "It's responding to ALL_MULTICAST_ROUTER group ping request \n";
			}
			
			if( firstHopV2 )
			{
				std::cout << "=> Path to first hop router is working for ASM, router responds to  IGMPv2 requests. \n";
			}
			

			if( routerSupportsV3 )
			{
				std::cout << "=> First hop router supports Source Specific Multicast (IGMP v3). "
					         "It's responding to ALL_SSM_ROUTER group ping request \n";
			}

			
			
			if( firstHopV3 )
			{
				std::cout << "=> Path to first hop router is working for SSM, router responds to IGMPv3 requests. \n";
			}
			else
			{
				if( hostSupportsV3 && routerSupportsV3 )
				{
					std::cout << "=> Path to first hop router is not setup for SSM, router does not respond to IGMPv3 requests. "
						         " Most probably it is a Switch related problem.  \n";
				}
			}

			if( routerSupportsPIM )
			{
				std::cout << "=> First hop router supports PIM - SM multicast routing protocol. "
							 "It's responding to ALL_PIMSM_ROUTER group ping request. \n";
			}
			
			if( routerPIMActive )
			{
				std::cout << "=> PIM-SM is active at first hop router and responds to Hello message. "
					         "The reponders are "
						  << routerAddress
						  << std::endl;
			}
			
			if( asmRoutingSuccess )
			{
				std::cout << "=> ASM is working for the 'Beacon Group', "
					         "RTP packets from members were received. \n";
			}
			else
			{
				if( hostSupportsV2 && routerSupportsV2 && firstHopV2 && routerSupportsPIM )
				{
					std::cout << "=> Path to first hop router seems to be working for ASM, but not receiving traffic from "
                                 "the Internet. This is likely to be a problem beyond the first-hop router. \n";
				}
			}

			if( ssmRoutingSuccess )
			{
				std::cout << "=> SSM is functioning for Beacon Group. "
					         " Receiving RTP packets from members of SSM beacon group. \n";
			}
			else
			{
				if( hostSupportsV3 && routerSupportsV3 && firstHopV3 && routerSupportsPIM )
				{
					std::cout << "=> Path to first hop router seems to be working for SSM, but not receiving traffic from "
                                 "the Internet. This is likely to be a problem beyond the first-hop router. \n";
				}
			}
		
            bool rerunTest = false;
            bool validInput = false;
            
			while( !validInput )
            {
                std::cout << "\n +> To exit enter 'q' , to re run the test enter 'y' \n";

			    std::string input;

			    std::cin >> input;

			    if( input == "q" )
			    {
				    validInput = true;
			    }
                else if( input == "y" )
                {
                    validInput = true;
                    rerunTest = true;
                }
            }

            if( !rerunTest )
            {
                break;
            }
		}
	}
    catch( MD_SocketException& exp )
    {
        std::cout << "Caught Exception:" << exp.Display() << std::endl;
    }



    return 0;
}
